@@ -36,8 +36,10 @@ import logging
 from gi.repository import Gtk, Gio, GLib, Gdk, GObject
 
 from gnomemusic import log
+from gnomemusic.coremodel import CoreModel
+from gnomemusic.coreselection import CoreSelection
 from gnomemusic.inhibitsuspend import InhibitSuspend
-from gnomemusic.mpris import MediaPlayer2Service
+from gnomemusic.mpris import MPRIS
 from gnomemusic.pauseonsuspend import PauseOnSuspend
 from gnomemusic.player import Player
 from gnomemusic.widgets.aboutdialog import AboutDialog
@@ -53,7 +55,6 @@ class Application(Gtk.Application):
         super().__init__(
             application_id=application_id,
             flags=Gio.ApplicationFlags.FLAGS_NONE)
-
         self.props.resource_base_path = "/org/gnome/Music"
         GLib.set_application_name(_("Music"))
         GLib.set_prgname(application_id)
@@ -62,6 +63,9 @@ class Application(Gtk.Application):
         self._init_style()
         self._window = None
 
+        self._coreselection = CoreSelection()
+        self._coremodel = CoreModel(self._coreselection)
+
         self._settings = Gio.Settings.new('org.gnome.Music')
         self._player = Player(self)
 
@@ -96,6 +100,26 @@ class Application(Gtk.Application):
         """
         return self._settings
 
+    @GObject.Property(
+        type=CoreModel, flags=GObject.ParamFlags.READABLE)
+    def coremodel(self):
+        """Get class providing all listmodels.
+
+        :returns: List model provider class
+        :rtype: CoreModel
+        """
+        return self._coremodel
+
+    @GObject.Property(
+        type=CoreSelection, flags=GObject.ParamFlags.READABLE)
+    def coreselection(self):
+        """Get selection object.
+
+        :returns: Object containing all selection info
+        :rtype: CoreSelection
+        """
+        return self._coreselection
+
     @log
     def _build_app_menu(self):
         action_entries = [
@@ -137,9 +161,6 @@ class Application(Gtk.Application):
             self._window.set_default_icon_name(self.props.application_id)
             if self.props.application_id == 'org.gnome.MusicDevel':
                 self._window.get_style_context().add_class('devel')
-            MediaPlayer2Service(self)
+            MPRIS(self)
 
-        # gtk_window_present does not work on Wayland.
-        # Use gtk_present_with_time as a workaround instead.
-        # See https://gitlab.gnome.org/GNOME/gtk/issues/624#note_10996
-        self._window.present_with_time(GLib.get_monotonic_time() / 1000)
+        self._window.present()
