@@ -22,10 +22,9 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from gi.repository import GdkPixbuf, GObject, Gtk
+from gi.repository import GObject, Gtk
 
 from gnomemusic import log
-from gnomemusic.grilo import grilo
 from gnomemusic.widgets.starhandlerwidget import StarHandlerWidget
 
 
@@ -54,20 +53,6 @@ class BaseView(Gtk.Stack):
 
         self._grid = Gtk.Grid(orientation=Gtk.Orientation.HORIZONTAL)
         self._offset = 0
-        self.model = Gtk.ListStore(
-            GObject.TYPE_STRING,
-            GObject.TYPE_STRING,
-            GObject.TYPE_STRING,
-            GObject.TYPE_STRING,
-            GdkPixbuf.Pixbuf,
-            GObject.TYPE_OBJECT,
-            GObject.TYPE_BOOLEAN,
-            GObject.TYPE_INT,
-            GObject.TYPE_STRING,
-            GObject.TYPE_INT,
-            GObject.TYPE_BOOLEAN,
-            GObject.TYPE_INT
-        )
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
 
         # Setup the main view
@@ -78,7 +63,7 @@ class BaseView(Gtk.Stack):
 
         self._grid.add(self._box)
 
-        self._star_handler = StarHandlerWidget(self, 9)
+        self._star_handler = StarHandlerWidget(self, 6)
         self._window = window
         self._headerbar = window._headerbar
 
@@ -87,19 +72,17 @@ class BaseView(Gtk.Stack):
 
         self.add(self._grid)
         self.show_all()
-        self._view.hide()
+        # self._view.hide()
 
         self._init = False
-        grilo.connect('ready', self._on_grilo_ready)
-        self.connect('notify::selection-mode', self._on_selection_mode_changed)
-        grilo.connect('changes-pending', self._on_changes_pending)
+        self._selection_mode_id = self.connect(
+            "notify::selection-mode", self._on_selection_mode_changed)
 
         self.bind_property(
             'selection-mode', self._window, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL)
 
-        if (grilo.tracker is not None
-                and not self._init):
+        if not self._init:
             self._on_grilo_ready()
 
     @log
@@ -135,10 +118,6 @@ class BaseView(Gtk.Stack):
         if not self.props.selection_mode:
             self.unselect_all()
 
-    @log
-    def _retrieval_finished(self, klass):
-        self.model[klass.iter][4] = klass.pixbuf
-
     @log
     def _on_item_activated(self, widget, id, path):
         pass
@@ -146,28 +125,3 @@ class BaseView(Gtk.Stack):
     @log
     def get_selected_songs(self, callback):
         callback([])
-
-    @log
-    def _set_selection(self, value, parent=None):
-        count = 0
-        itr = self.model.iter_children(parent)
-        while itr is not None:
-            if self.model.iter_has_child(itr):
-                count += self._set_selection(value, itr)
-            if self.model[itr][5] is not None:
-                self.model[itr][6] = value
-                count += 1
-            itr = self.model.iter_next(itr)
-
-        return count
-
-    @log
-    def select_all(self):
-        """Select all the available songs."""
-        self.props.selected_items_count = self._set_selection(True)
-
-    @log
-    def unselect_all(self):
-        """Unselects all the selected songs."""
-        self._set_selection(False)
-        self.props.selected_items_count = 0
