@@ -27,10 +27,9 @@ from gi.repository import Gdk, GObject, Gtk
 
 from gnomemusic import log
 from gnomemusic.widgets.songwidget import SongWidget
-import gnomemusic.utils as utils
 
 
-class DiscSongsFlowBox(Gtk.FlowBox):
+class DiscSongsFlowBox(Gtk.ListBox):
     """FlowBox containing the songs on one disc
 
     DiscSongsFlowBox allows setting the number of columns to
@@ -42,45 +41,11 @@ class DiscSongsFlowBox(Gtk.FlowBox):
         return '<DiscSongsFlowBox>'
 
     @log
-    def __init__(self, columns=1):
+    def __init__(self):
         """Initialize
-
-        :param int columns: The number of columns the widget uses
         """
         super().__init__(selection_mode=Gtk.SelectionMode.NONE)
 
-        self._columns = 1
-        self.props.columns = columns
-
-        self.get_style_context().add_class('discsongsflowbox')
-
-    @GObject.Property(type=int, minimum=1, default=1)
-    def columns(self):
-        """Number of columns for the song list
-
-        :returns: The number of columns
-        :rtype: int
-        """
-        return self._columns
-
-    @columns.setter
-    def columns(self, columns):
-        """Set the number of columns to use
-
-        :param int columns: The number of columns the widget uses
-        """
-        self._columns = columns
-
-        children_n = len(self.get_children())
-
-        if children_n % self._columns == 0:
-            max_per_line = children_n / self._columns
-        else:
-            max_per_line = int(children_n / self._columns) + 1
-
-        self.props.max_children_per_line = max_per_line
-        self.props.min_children_per_line = max_per_line
-
 
 @Gtk.Template(resource_path='/org/gnome/Music/ui/DiscBox.ui')
 class DiscBox(Gtk.Box):
@@ -92,14 +57,15 @@ class DiscBox(Gtk.Box):
     __gtype_name__ = 'DiscBox'
 
     _disc_label = Gtk.Template.Child()
-    _disc_songs_flowbox = Gtk.Template.Child()
+    # _disc_songs_flowbox = Gtk.Template.Child()
+    _list_box = Gtk.Template.Child()
 
     __gsignals__ = {
+        "ready": (GObject.SignalFlags.RUN_FIRST, None, ()),
         'selection-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'song-activated': (GObject.SignalFlags.RUN_FIRST, None, (Gtk.Widget,))
     }
 
-    columns = GObject.Property(type=int, minimum=1, default=1)
     selection_mode = GObject.Property(type=bool, default=False)
     selection_mode_allowed = GObject.Property(type=bool, default=True)
     show_disc_label = GObject.Property(type=bool, default=False)
@@ -111,19 +77,15 @@ class DiscBox(Gtk.Box):
         return '<DiscBox>'
 
     @log
-    def __init__(self, model=None):
+    def __init__(self, model):
         """Initialize
 
-        :param model: The TreeStore to use
+        :param model: The Gio.ListStore to use
         """
         super().__init__()
 
         self._model = model
-        self._model.connect('row-changed', self._model_row_changed)
 
-        self.bind_property(
-            'columns', self._disc_songs_flowbox, 'columns',
-            GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
             'show-disc-label', self._disc_label, 'visible',
             GObject.BindingFlags.SYNC_CREATE)
@@ -132,6 +94,10 @@ class DiscBox(Gtk.Box):
         self._selected_items = []
         self._songs = []
 
+        self._model.connect_after(
+            "items-changed", self._on_model_items_changed)
+        self._list_box.bind_model(self._model, self._create_widget)
+
     @log
     def set_disc_number(self, disc_number):
         """Set the dics number to display
@@ -141,96 +107,46 @@ class DiscBox(Gtk.Box):
         self._disc_label.props.label = _("Disc {}").format(disc_number)
         self._disc_label.props.visible = True
 
-    @log
-    def set_songs(self, songs):
-        """Songs to display
-
-        :param list songs: A list of Grilo media items to
-        add to the widget
-        """
-        for song in songs:
-            song_widget = self._create_song_widget(song)
-            self._disc_songs_flowbox.insert(song_widget, -1)
-            song.song_widget = song_widget
-
-    @log
     def get_selected_items(self):
         """Return all selected items
 
         :returns: The selected items:
         :rtype: A list if Grilo media items
         """
-        self._selected_items = []
-        self._disc_songs_flowbox.foreach(self._get_selected)
+        return []
 
-        return self._selected_items
-
-    @log
-    def _get_selected(self, child):
-        song_widget = child.get_child()
-
-        if song_widget.selected:
-            itr = song_widget.itr
-            self._selected_items.append(self._model[itr][5])
-
-    # FIXME: select all/none slow probably b/c of the row changes
-    # invocations, maybe workaround?
-    @log
     def select_all(self):
         """Select all songs"""
         def child_select_all(child):
             song_widget = child.get_child()
-            self._model[song_widget.itr][6] = True
+            song_widget.props.selected = True
 
-        self._disc_songs_flowbox.foreach(child_select_all)
+        self._list_box.foreach(child_select_all)
 
-    @log
     def select_none(self):
         """Deselect all songs"""
         def child_select_none(child):
             song_widget = child.get_child()
-            self._model[song_widget.itr][6] = False
+            song_widget.props.selected = False
 
-        self._disc_songs_flowbox.foreach(child_select_none)
+        self._list_box.foreach(child_select_none)
 
-    @log
-    def _create_song_widget(self, song):
-        """Helper function to create a song widget for a
-        single song
-
-        :param song: A Grilo media item
-        :returns: A complete song widget
-        :rtype: Gtk.EventBox
-        """
-        song_widget = SongWidget(song)
+    def _create_widget(self, coresong):
+        song_widget = SongWidget(coresong)
         self._songs.append(song_widget)
 
-        title = utils.get_media_title(song)
+        self.bind_property(
+            "selection-mode", song_widget, "selection-mode",
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
 
-        itr = self._model.append(None)
-
-        self._model[itr][0, 1, 2, 5, 6] = [title, '', '', song, False]
-
-        song_widget.itr = itr
-        song_widget.model = self._model
         song_widget.connect('button-release-event', self._song_activated)
-        song_widget.connect('selection-changed', self._on_selection_changed)
-
-        self.bind_property(
-            'selection-mode', song_widget, 'selection-mode',
-            GObject.BindingFlags.SYNC_CREATE)
-        self.bind_property(
-            'show-durations', song_widget, 'show-duration',
-            GObject.BindingFlags.SYNC_CREATE)
-        self.bind_property(
-            'show-favorites', song_widget, 'show-favorite',
-            GObject.BindingFlags.SYNC_CREATE)
-        self.bind_property(
-            'show-song-numbers', song_widget, 'show-song-number',
-            GObject.BindingFlags.SYNC_CREATE)
 
         return song_widget
 
+    def _on_model_items_changed(self, model, position, removed, added):
+        self.emit("ready")
+
     @log
     def _on_selection_changed(self, widget):
         self.emit('selection-changed')
@@ -249,33 +165,21 @@ class DiscBox(Gtk.Box):
                 and not self.props.selection_mode
                 and self.props.selection_mode_allowed):
             self.props.selection_mode = True
+            return
 
         (_, button) = event.get_button()
         if (button == Gdk.BUTTON_PRIMARY
                 and not self.props.selection_mode):
             self.emit('song-activated', widget)
 
-        if self.props.selection_mode:
-            itr = widget.itr
-            self._model[itr][6] = not self._model[itr][6]
-
-        return True
-
-    @log
-    def _model_row_changed(self, model, path, itr):
-        if (not self.props.selection_mode
-                or not model[itr][5]):
-            return
-
-        song_widget = model[itr][5].song_widget
-        selected = model[itr][6]
-        if selected != song_widget.props.selected:
-            song_widget.props.selected = selected
+        # FIXME: Need to ignore the event from the checkbox.
+        # if self.props.selection_mode:
+        #     widget.props.selected = not widget.props.selected
 
         return True
 
 
-class DiscListBox(Gtk.Box):
+class DiscListBox(Gtk.ListBox):
     """A ListBox widget containing all discs of a particular
     album
     """
@@ -293,28 +197,13 @@ class DiscListBox(Gtk.Box):
     @log
     def __init__(self):
         """Initialize"""
-        super().__init__(orientation=Gtk.Orientation.VERTICAL)
+        super().__init__()
 
+        self.props.valign = Gtk.Align.START
         self._selection_mode = False
         self._selected_items = []
 
-    @log
-    def add(self, widget):
-        """Insert a DiscBox widget"""
-        super().add(widget)
-        widget.connect('selection-changed', self._on_selection_changed)
-
-        self.bind_property(
-            'selection-mode', widget, 'selection-mode',
-            GObject.BindingFlags.BIDIRECTIONAL
-            | GObject.BindingFlags.SYNC_CREATE)
-        self.bind_property(
-            'selection-mode-allowed', widget, 'selection-mode-allowed',
-            GObject.BindingFlags.SYNC_CREATE)
-
-    @log
-    def _on_selection_changed(self, widget):
-        self.emit('selection-changed')
+        self.get_style_context().add_class("disc-list-box")
 
     @log
     def get_selected_items(self):
@@ -336,7 +225,7 @@ class DiscListBox(Gtk.Box):
     def select_all(self):
         """Select all songs"""
         def child_select_all(child):
-            child.select_all()
+            child.get_child().select_all()
 
         self.foreach(child_select_all)
 
@@ -344,7 +233,7 @@ class DiscListBox(Gtk.Box):
     def select_none(self):
         """Deselect all songs"""
         def child_select_none(child):
-            child.select_none()
+            child.get_child().select_none()
 
         self.foreach(child_select_none)
 
@@ -367,3 +256,9 @@ class DiscListBox(Gtk.Box):
             return
 
         self._selection_mode = value
+
+        def set_selection_mode(child):
+            print("set selection mode on", child)
+            child.props.selection_mode = value
+
+        self.foreach(set_selection_mode)
