@@ -32,7 +32,6 @@ from gi.repository import Gd, GLib, GObject, Gtk, Pango
 from gi.repository.Gd import TaggedEntry  # noqa: F401
 
 from gnomemusic import log
-from gnomemusic.grilo import grilo
 from gnomemusic.search import Search
 
 
@@ -118,7 +117,7 @@ class SourceManager(BaseManager):
         self.values.append(['grl-tracker-source', _("Local"), ''])
         self.props.default_value = 2
 
-        grilo.connect('new-source-added', self._add_new_source)
+        # grilo.connect('new-source-added', self._add_new_source)
 
     @log
     def fill_in_values(self, model):
@@ -144,8 +143,8 @@ class SourceManager(BaseManager):
 
         Adds available Grilo sources to the internal model.
         """
-        for id_ in grilo.props.sources:
-            self._add_new_source(None, grilo.props.sources[id_])
+        # for id_ in grilo.props.sources:
+        #     self._add_new_source(None, grilo.props.sources[id_])
 
     @GObject.Property
     def active(self):
@@ -159,8 +158,8 @@ class SourceManager(BaseManager):
         # https://gitlab.gnome.org/GNOME/gnome-music/snippets/31
         super(SourceManager, self.__class__).active.fset(self, selected_id)
 
-        src = grilo.sources[selected_id] if selected_id != 'all' else None
-        grilo.search_source = src
+        # src = grilo.sources[selected_id] if selected_id != 'all' else None
+        # grilo.search_source = src
 
 
 @Gtk.Template(resource_path="/org/gnome/Music/ui/FilterView.ui")
@@ -317,7 +316,8 @@ class DropDown(Gtk.Revealer):
 
     @log
     def _is_tracker(self, grilo_id):
-        return grilo_id == "grl-tracker-source"
+        return True
+        # return grilo_id == "grl-tracker-source"
 
 
 @Gtk.Template(resource_path="/org/gnome/Music/ui/SearchBar.ui")
@@ -337,10 +337,11 @@ class SearchBar(Gtk.SearchBar):
         return '<SearchBar>'
 
     @log
-    def __init__(self):
+    def __init__(self, application):
         """Initialize the SearchBar"""
         super().__init__()
 
+        self._application = application
         self._timeout = None
 
         self._dropdown = DropDown()
@@ -375,15 +376,17 @@ class SearchBar(Gtk.SearchBar):
         self._timeout = None
 
         search_term = self._search_entry.get_text()
-        if grilo.search_source:
-            fields_filter = self._dropdown.search_manager.active
-        else:
-            fields_filter = 'search_all'
+        # if grilo.search_source:
+        #     fields_filter = self._dropdown.search_manager.active
+        # else:
+        #     fields_filter = 'search_all'
 
         if search_term != "":
             self.props.stack.set_visible_child_name('search')
-            view = self.props.stack.get_visible_child()
-            view.set_search_text(search_term, fields_filter)
+
+            self._application._coremodel.search(search_term)
+            # view = self.props.stack.get_visible_child()
+            # view.set_search_text(search_term, fields_filter)
         else:
             self._set_error_style(False)
 
