@@ -1,11 +1,4 @@
-# Copyright (c) 2013 Eslam Mostafa <cseslam@gmail.com>
-# Copyright (c) 2013 Vadim Rutkovsky <vrutkovs@redhat.com>
-# Copyright (c) 2013 Sai Suman Prayaga <suman.sai14@gmail.com>
-# Copyright (c) 2013 Arnel A. Borja <kyoushuu@yahoo.com>
-# Copyright (c) 2013 Shivani Poddar <shivani.poddar92@gmail.com>
-# Copyright (c) 2013 Manish Sinha <manishsinha@ubuntu.com>
-# Copyright (c) 2013 Seif Lotfy <seif@gmail.com>
-# Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
+# Copyright 2019 The GNOME Music developers
 #
 # GNOME Music is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,11 +26,9 @@ from gi.repository import Gtk, Gdk, Gio, GLib, GObject
 from gettext import gettext as _
 
 from gnomemusic import log
-from gnomemusic.grilo import grilo
+from gnomemusic.gstplayer import Playback
 from gnomemusic.mediakeys import MediaKeys
 from gnomemusic.player import RepeatMode
-from gnomemusic.playlists import Playlists
-from gnomemusic.query import Query
 from gnomemusic.search import Search
 from gnomemusic.utils import View
 from gnomemusic.views.albumsview import AlbumsView
@@ -45,26 +36,33 @@ from gnomemusic.views.artistsview import ArtistsView
 from gnomemusic.views.emptyview import EmptyView
 from gnomemusic.views.searchview import SearchView
 from gnomemusic.views.songsview import SongsView
-from gnomemusic.views.playlistview import PlaylistView
+from gnomemusic.views.playlistsview import PlaylistsView
 from gnomemusic.widgets.headerbar import HeaderBar
-from gnomemusic.widgets.notificationspopup import NotificationsPopup
+from gnomemusic.widgets.notificationspopup import NotificationsPopup  # noqa
 from gnomemusic.widgets.playertoolbar import PlayerToolbar
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
 from gnomemusic.widgets.searchbar import SearchBar
-from gnomemusic.widgets.selectiontoolbar import SelectionToolbar
+from gnomemusic.widgets.selectiontoolbar import SelectionToolbar  # noqa: F401
 from gnomemusic.windowplacement import WindowPlacement
 
 import logging
 logger = logging.getLogger(__name__)
 
-playlists = Playlists.get_default()
-
 
+@Gtk.Template(resource_path="/org/gnome/Music/ui/Window.ui")
 class Window(Gtk.ApplicationWindow):
 
+    __gtype_name__ = "Window"
+
     selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
+    notifications_popup = Gtk.Template.Child()
+    _box = Gtk.Template.Child()
+    _overlay = Gtk.Template.Child()
+    _selection_toolbar = Gtk.Template.Child()
+    _stack = Gtk.Template.Child()
+
     def __repr__(self):
         return '<Window>'
 
@@ -76,6 +74,12 @@ class Window(Gtk.ApplicationWindow):
         """
         super().__init__(application=app, title=_("Music"))
 
+        # Hack
+        self._app = app
+
+        self._app._coreselection.bind_property(
+            "selected-items-count", self, "selected-items-count")
+
         self._settings = app.props.settings
         self.add_action(self._settings.create_action('repeat'))
         select_all = Gio.SimpleAction.new('select_all', None)
@@ -93,42 +97,17 @@ class Window(Gtk.ApplicationWindow):
 
         self._player = app.props.player
 
-        self.notifications_popup = NotificationsPopup()
         self._setup_view()
 
         MediaKeys(self._player, self)
 
-        grilo.connect('changes-pending', self._on_changes_pending)
-
-    @log
-    def _on_changes_pending(self, data=None):
-        # FIXME: This is not working right.
-        def songs_available_cb(available):
-            view_count = len(self.views)
-            if (available
-                    and view_count == 1):
-                self._switch_to_player_view()
-            elif (not available
-                    and not self.props.selection_mode
-                    and view_count != 1):
-                self._stack.disconnect(self._on_notify_model_id)
-                self.disconnect(self._key_press_event_id)
-
-                for i in range(View.ALBUM, view_count):
-                    view = self.views.pop()
-                    view.destroy()
-
-                self._switch_to_empty_view()
-
-        grilo.songs_available(songs_available_cb)
-
     @log
     def _setup_view(self):
-        self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-
         self._search = Search()
+        self._searchbar = SearchBar(self._app)
+        self._searchbar.props.stack = self._stack
         self._headerbar = HeaderBar()
-        self._searchbar = SearchBar()
+
         self._search.bind_property(
             "search-mode-active", self._headerbar, "search-mode-active",
             GObject.BindingFlags.BIDIRECTIONAL
@@ -140,32 +119,29 @@ class Window(Gtk.ApplicationWindow):
             "state", self._searchbar, "search-state",
             GObject.BindingFlags.SYNC_CREATE)
 
-        self._player_toolbar = PlayerToolbar(self._player, self)
-        selection_toolbar = SelectionToolbar()
-        self.views = [None] * len(View)
-        self._stack = Gtk.Stack(
-            transition_type=Gtk.StackTransitionType.CROSSFADE,
-            transition_duration=100,
-            homogeneous=False,
-            visible=True,
-            can_focus=False)
+        self._player_toolbar = PlayerToolbar()
+        self._player_toolbar.props.player = self._player
 
-        self._searchbar.props.stack = self._stack
         self._headerbar.connect(
             'back-button-clicked', self._switch_back_from_childview)
 
-        self.connect('notify::selection-mode', self._on_selection_mode_changed)
         self.bind_property(
             'selected-items-count', self._headerbar, 'selected-items-count')
         self.bind_property(
-            'selected-items-count', selection_toolbar, 'selected-items-count')
+            "selected-items-count", self._selection_toolbar,
+            "selected-items-count")
         self.bind_property(
             'selection-mode', self._headerbar, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL
             | GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
-            'selection-mode', selection_toolbar, 'visible',
-            GObject.BindingFlags.SYNC_CREATE)
+            "selection-mode", self._player_toolbar, "visible",
+            GObject.BindingFlags.INVERT_BOOLEAN)
+        self.bind_property(
+            "selection-mode", self._selection_toolbar, "visible")
+        self.connect("notify::selection-mode", self._on_selection_mode_changed)
+
+        self.views = [Gtk.Box()] * len(View)
         # Create only the empty view at startup
         # if no music, switch to empty view and hide stack
         # if some music is available, populate stack with mainviews,
@@ -177,37 +153,27 @@ class Window(Gtk.ApplicationWindow):
         # bottom line of the searchbar
         self._stack.get_style_context().add_class('background')
 
-        self._overlay = Gtk.Overlay()
-        self._overlay.add(self._stack)
         # FIXME: Need to find a proper way to do this.
         self._overlay.add_overlay(self._searchbar._dropdown)
-        self._overlay.add_overlay(self.notifications_popup)
-        self.set_titlebar(self._headerbar)
-        self._box.pack_start(self._searchbar, False, False, 0)
-        self._box.pack_start(self._overlay, True, True, 0)
-        self._box.pack_start(self._player_toolbar, False, False, 0)
-        self._box.pack_start(selection_toolbar, False, False, 0)
-        self.add(self._box)
 
-        selection_toolbar.connect(
+        self._box.pack_start(self._searchbar, False, False, 0)
+        self._box.reorder_child(self._searchbar, 0)
+        self._box.pack_end(self._player_toolbar, False, False, 0)
+
+        self.set_titlebar(self._headerbar)
+
+        self._selection_toolbar.connect(
             'add-to-playlist', self._on_add_to_playlist)
         self._search.connect("notify::state", self._on_search_state_changed)
 
         self._headerbar.props.state = HeaderBar.State.MAIN
         self._headerbar.show()
-        self._overlay.show()
-        self._player_toolbar.show_all()
-        self._box.show()
-        self.show()
 
-        def songs_available_cb(available):
-            if available:
-                self._switch_to_player_view()
-            else:
-                self._switch_to_empty_view()
+        self._app.props.coremodel.connect(
+            "notify::songs-available", self._on_songs_available)
 
-        if Query().music_folder:
-            grilo.songs_available(songs_available_cb)
+        if self._app.props.coremodel.props.songs_available:
+            self._switch_to_player_view()
         else:
             self._switch_to_empty_view()
 
@@ -215,15 +181,25 @@ class Window(Gtk.ApplicationWindow):
     def _switch_to_empty_view(self):
         did_initial_state = self._settings.get_boolean('did-initial-state')
 
-        if not grilo.props.tracker_available:
+        # FIXME: Tracker just checks for TrackerWrapper right now.
+        # It should also check for the viability of certain queries to
+        # make sure we have a recent version available.
+        if not self._app.props.coremodel._grilo.props.tracker_available:
             self.views[View.EMPTY].props.state = EmptyView.State.NO_TRACKER
         elif did_initial_state:
             self.views[View.EMPTY].props.state = EmptyView.State.EMPTY
         else:
+            # FIXME: On switch back this view does not show properly.
             self.views[View.EMPTY].props.state = EmptyView.State.INITIAL
 
         self._headerbar.props.state = HeaderBar.State.EMPTY
 
+    def _on_songs_available(self, klass, value):
+        if self._app.props.coremodel.props.songs_available:
+            self._switch_to_player_view()
+        else:
+            self._switch_to_empty_view()
+
     @log
     def _switch_to_player_view(self):
         self._settings.set_boolean('did-initial-state', True)
@@ -239,6 +215,8 @@ class Window(Gtk.ApplicationWindow):
         self._btn_ctrl.props.button = 8
         self._btn_ctrl.connect("pressed", self._on_back_button_pressed)
 
+        self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
+
         # FIXME: In case Grilo is already initialized before the views
         # get created, they never receive a 'ready' signal to trigger
         # population. To fix this another check was added to baseview
@@ -248,13 +226,6 @@ class Window(Gtk.ApplicationWindow):
         # to window._stack. For this to succeed, the stack needs to be
         # filled with something: Gtk.Box.
         # This is a bit of circular logic that needs to be fixed.
-        self.views[View.ALBUM] = Gtk.Box()
-        self.views[View.ARTIST] = Gtk.Box()
-        self.views[View.SONG] = Gtk.Box()
-        self.views[View.PLAYLIST] = Gtk.Box()
-        self.views[View.SEARCH] = Gtk.Box()
-
-        self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
         self._headerbar.props.state = HeaderBar.State.MAIN
         self._headerbar.props.stack = self._stack
         self._searchbar.show()
@@ -262,7 +233,7 @@ class Window(Gtk.ApplicationWindow):
         self.views[View.ALBUM] = AlbumsView(self, self._player)
         self.views[View.ARTIST] = ArtistsView(self, self._player)
         self.views[View.SONG] = SongsView(self, self._player)
-        self.views[View.PLAYLIST] = PlaylistView(self, self._player)
+        self.views[View.PLAYLIST] = PlaylistsView(self, self._player)
         self.views[View.SEARCH] = SearchView(self, self._player)
 
         selectable_views = [View.ALBUM, View.ARTIST, View.SONG, View.SEARCH]
@@ -488,31 +459,27 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
-        if self.props.selection_mode:
+        if (not self.props.selection_mode
+                and self._player.state == Playback.STOPPED):
             self._player_toolbar.hide()
-        elif self._player.props.playing:
-            self._player_toolbar.show()
-        if not self.props.selection_mode:
-            self._on_changes_pending()
 
     @log
     def _on_add_to_playlist(self, widget):
         if self._stack.get_visible_child() == self.views[View.PLAYLIST]:
             return
 
-        def callback(selected_songs):
-            if len(selected_songs) < 1:
-                return
+        selected_songs = self._app._coreselection.props.selected_items
 
-            playlist_dialog = PlaylistDialog(
-                self, self.views[View.PLAYLIST].pls_todelete)
-            if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
-                playlists.add_to_playlist(
-                    playlist_dialog.get_selected(), selected_songs)
-            self.props.selection_mode = False
-            playlist_dialog.destroy()
+        if len(selected_songs) < 1:
+            return
 
-        self._stack.get_visible_child().get_selected_songs(callback)
+        playlist_dialog = PlaylistDialog(self)
+        if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
+            playlist = playlist_dialog.props.selected_playlist
+            playlist.add_songs(selected_songs)
+
+        self.props.selection_mode = False
+        playlist_dialog.destroy()
 
     @log
     def set_player_visible(self, visible):
